-- Template dump tools

ROOT_TAG = "ObjectTemplate"

function DumpToTemplate(gameObj,name,skipState)
	DebugMessage("Called")
	local dump = xml.new(ROOT_TAG)
	dump["xmlns:xsi"] = "http://www.w3.org/2001/XMLSchema-instance"
	dump["xmlns:xsd"] = "http://www.w3.org/2001/XMLSchema"

	dump:append("Name")[1] = gameObj:GetName()
	dump:append("ClientId")[1] = gameObj:GetIconId()
	dump:append("Hue")[1] = gameObj:GetHue()
	dump:append("ScaleModifier")[1] = gameObj:GetScale()
	DebugMessage("Base modifiers done")

	if not skipState then
		local tags = gameObj:GetAllSharedObjectProperties()
		DebugTable(tags)
		for name,tag in pairs(tags) do -- Dump all shared object properties
			--local prop = gameObj:GetSharedObjectProperty(tag)
			local entry = dump:append("SharedStateEntry")
			entry["name"] = name
			if type(tag) == "number" then
				local int,frac = math.modf(tag)
				if frac == 0 then
					entry["type"] = "int"
				else
					entry["type"] = "double"
				end
			elseif type(tag) == "string" then
				entry["type"] = "string"
			elseif type(tag) == "boolean" then
				entry["type"] = "bool"
			else
				entry["type"] = "INVALID"
			end

			entry["value"] = tag
		end
	end
	--DebugMessage("ObjectTags done")

	local objVars = gameObj:GetAllObjVars()
	local parsedVars = {}
	local modules = gameObj:GetAllModules()
	local modDump = dump:append("ScriptEngineComponent")

	for i,name in pairs(modules) do
		local tag = modDump:append("LuaModule")
		tag.Name = name
		local parser = InitializerParsers[name]
		if parser ~= nil then
			tag:append("Initializer")[1] = parser(gameObj, objVars, parsedVars)
		end
	end
	--DebugMessage("Initializers done")

	-- Mobile component
	if gameObj:IsMobile() then
		local mobileData = dump:append("MobileComponent")
		mobileData:append("BaseRunSpeed")[1] = gameObj:GetBaseMoveSpeed()
		mobileData:append("MobileType")[1] = gameObj:GetMobileType()
	end

	-- Unparsed objvars
	local varDump = dump:append("ObjectVariableComponent")
	for name,data in pairs(objVars) do
		if parsedVars[name] then goto continue end

		local entry
		local fail = false
		if type(data) == "string" then
			entry = varDump:append("StringVariable")
		elseif type(data) == "number" then
			entry = varDump:append("DoubleVariable")
		elseif type(data) == "boolean" then
			entry = varDump:append("BoolVariable")
		elseif type(data) == "userdata" then
			if GetUserdataType(data) == "Loc" then
				fail = true -- override default mechanism
				entry = varDump:append("LocVariable")
				entry.Name = name
				entry[1] = tostring(data.X)..","..tostring(data.Y)..","..tostring(data.Z)
			end
		else
			fail = true
			DebugMessage("Can't dump table ObjVar "..name..", please write initializer parser!")
		end

		if not fail then
			entry.Name = name
			entry[1] = data
		end

		::continue::
	end
	--DebugMessage("Rest of ObjVars done")

	DebugMessage("Saving to file <"..name..">")
	local str = dump:str():gsub("&quot;", "\"")
	local file = io.open(name, "w+")
	file:write(str)
	file:close()
	DebugMessage("Saved")
end

function SerializeTableRaw(name, table, last, indent)
	local strs = {}
	if name ~= "" then
		strs[1] = indent..name.." = {"
	else
		strs[1] = indent.."{"
	end

	local inner = indent.."\t" -- Inner indentation
	for k,v in pairs(table) do
		if type(k) == "number" then
			if type(v) == "table" then
				strs[#strs+1] = TableConcat(SerializeTableRaw("", v, false, inner), "\n")
			elseif type(v) == "string" then
				strs[#strs+1] = inner.."\""..v.."\","
			else
				strs[#strs+1] = inner..tostring(v)..","
			end
		else
			if type(v) == "table" then
				strs[#strs+1] = TableConcat(SerializeTableRaw(k, v, false, inner), "\n")
			elseif type(v) == "string" then
				strs[#strs+1] = inner..k.." = ".."\""..v.."\","
			else
				strs[#strs+1] = inner..k.." = "..tostring(v)..","
			end
		end
	end

	if last then
		strs[#strs+1] = indent.."}"
	else
		strs[#strs+1] = indent.."},"
	end

	return strs
end

function SerializeTable(name, table)
	local strs = SerializeTableRaw(name, table, true, "")
	return TableConcat(strs, "\n")
end

function TableConcat(table,separator)
	local ret = ""
	for i,str in ipairs(table) do
		ret = ret..str..separator
	end

	return ret
end

function GetEquipEntry(obj, slot)
	local item = obj:GetEquippedObject(slot)
	if not item then return nil end

	return {{item:GetCreationTemplateId(), item:GetHue()}}
end

InitializerParsers = {}

function ParseModularAI(gameObj, objVars, parsed)
	local initializer = {
		Modules = objVars.AIModules,
		Traits = objVars.RelTraits,
		Concerns = objVars.RelConcerns,
		Stats = {
			Str = objVars.AttrStr,
			Agi = objVars.AttrAgi,
			Int = objVars.AttrInt
		},
		EquipTable = {
			RightHand = GetEquipEntry(gameObj, "RightHand"),
			LeftHand = GetEquipEntry(gameObj, "LeftHand"),
			Chest = GetEquipEntry(gameObj, "Chest"),
			Legs = GetEquipEntry(gameObj, "Legs"),
			Head = GetEquipEntry(gameObj, "Head"),
			BodyPartHair = GetEquipEntry(gameObj, "BodyPartHair"),
			BodyPartHead = GetEquipEntry(gameObj, "BodyPartHead")
		},
		DialogueData = objVars.DialogueData
	}
	parsed.AIModules = true
	parsed.RelTraits = true
	parsed.RelConcerns = true
	parsed.AttrStr = true
	parsed.AttrAgi = true
	parsed.AttrInt = true
	parsed.DialogueData = true
	parsed.mobileInitialized = true -- Otherwise, no clothing...
	parsed.MobileTeamType = true -- This makes mobile type not get parsed
	parsed.Sleeping = true -- Come on...
	parsed.BaseMovementSpeed = true -- Mobile component
	parsed.IsMerchant = true -- Use merchant AI module, or something?
	parsed.TemplateName = true -- See the Name field in template...
	parsed.SpawnScale = true -- See Scale field
	parsed.CurrentState = true -- We just don't store state machine to templates!
	parsed.SpawnFacing = true
	parsed.lootable = true -- Nope!
	parsed.LastKillerName = true -- Seriously...
	parsed.LastKilledRegion = true
	parsed.LastKilledTicks = true

	return SerializeTable("", initializer)
end
InitializerParsers["ai_modular"] = ParseModularAI

function ParseWeaponBase(gameObj, objVars, parsed)
	local initializer = {
		BaseMods = objVars.WeaponBaseMods
	}
	parsed.WeaponBaseMods = true

	return SerializeTable("", initializer)
end
InitializerParsers["weapon_base"] = ParseWeaponBase