require 'base_mobile'
require 'incl_dialogwindow'
require 'incl_PvPEvent'

Warchief = 
{

    -- Used to capture warchief team. Set by derived class
    MyTeam = nil,

    --Collection of phrases to speak
    ThingsToSay = { "Come join the ranks adventurer.", 
                    "Do or do not, there is no try.", 
                    "Greetings" }     
 
}

shieldTemplates = 
{ 
    Necromancer="shield_large_necromancer", 
    Paladin="shield_large_paladin", 
    Witch="shield_large_witch"
    Slayer="shield_large_slayer"
}

spawnLocations = 
{ 
    Necromancer=Loc(72,0,122), 
    Paladin=Loc(-160.64,0,124.44), 
    Witch=Loc(-92.7,0,-72.39)
    Slayer=Loc(00,0,000)
}

function GiveShield(player)
  local shieldTemplate = shieldTemplates[Warchief.MyTeam]

  local shieldObj = player:GetEquippedObject("LeftHand")
  if( shieldObj ~= nil and shieldObj:GetCreationTemplateId() == shieldTemplate) then
    return
  end

  local backpackObj = player:GetEquippedObject("Backpack")  
  if( backpackObj ~= nil and shieldTemplate ~= nil ) then
    -- if we don't already have a shield
    if( #FindObjects(SearchMulti{
                          SearchContainer(backpackObj),
                          SearchTemplate(shieldTemplate)
                        }) == 0 ) then
      CreateObjInBackpack(player,shieldTemplate)
    end
  end
end

function PlayerRequestTeam(player)
 
  local curTeam = player:GetObjVar("MobileTeamType")

  if( player:IsImmortal() and not(player:IsGod()) ) then
    this:NpcSpeech("Immortals and Gods have no place on this battlefield.")
    return
  end

  ClientDialog.Show{
    TargetUser = player,
    DialogId = tostring(player.Id)..Warchief.MyTeam,
    TitleStr = "PvP Queue",
    DescStr = "Do you wish to join clan "..Warchief.MyTeam.." to compete for glory?",
    Button1Str = "Accept",
    Button2Str = "Decline"
    }

end

function PlayerInitTeam(player)
  player:SetObjVar("MobileTeamType", Warchief.MyTeam)
  player:SetObjVar("SpawnPosition",spawnLocations[Warchief.MyTeam])
  player:AddModule("incl_PvPPlayer")
  
  player:SendMessage("UpdateName")

  GiveShield(player)
        
  this:NpcSpeech(player:GetName().." is now a soldier for "..Warchief.MyTeam..".") 
  this:NpcSpeech("To leave your team just say you [FFF544][quit][-]. I have given you a shield.  If you wear it in battle, the "..Warchief.MyTeam.." spirits will protect you.")   
end

function PlayerQuitTeam(player)
  local team = player:GetObjVar("MobileTeamType")
  if( team == nil ) then return end

  if( GetPvPGameState() == "Match" ) then
    this:NpcSpeech("You can not abandon your army during a match!")
    return
  end

  player:SetObjVar("MobileTeamType",nil)
  player:SetObjVar("SpawnPosition",nil)

  if(player:HasModule("incl_PvPPlayer")) then
    player:DelModule("incl_PvPPlayer")
  end

  player:SendMessage("UpdateName")

  this:NpcSpeech("You have been banished from the "..Warchief.MyTeam.." clan. May science have mercy on your soul.")
end

function CollectItems(user)
  local loyaltyVal = 0
  local backpackObj = user:GetEquippedObject("Backpack")
  if( backpackObj ~= nil ) then
    local packObjects = backpackObj:GetContainedObjects()
    for index, packObj in pairs(packObjects) do 
      if( packObj:GetCreationTemplateId() == "player_head" ) then
        loyaltyVal = loyaltyVal + 20
        packObj:Destroy()
      elseif( packObj:GetCreationTemplateId() == "teeth_ogre" ) then
        loyaltyVal = loyaltyVal + (packObj:GetStackCount() * 40)
        packObj:Destroy()
      elseif( packObj:GetCreationTemplateId() == "cultist_relic" ) then
        loyaltyVal = loyaltyVal + (packObj:GetStackCount() * 20)
        packObj:Destroy()
      end
    end
  end

  if( loyaltyVal > 0 ) then
    AddLoyalty(user,Warchief.MyTeam,loyaltyVal)
    this:NpcSpeech("Well done "..user:GetName()..". You have been granted "..loyaltyVal.." loyalty for your service.")
  else
    this:NpcSpeech("If you bring me the heads of your fallen enemies, I reward you with loyalty. I will also grant loyalty for ogre teeth and cultist relics.")
  end
end

function Warchief.Init()
  this:SetObjVar("noLoot", 1)
end

function HandlePlayerSpeech(speaker,speech)
  if(string.find(speech, "quit") ~= nil) then
    Warchief.PlayerQuitTeam(speaker)
  end
end

function HandleDialogResult(user,dialogId,buttonId)
  if (buttonId == 0) then   
     PlayerInitTeam(user)
  else
    return false
  end 
end

function HandleInteract(user)
    local userTeam = user:GetObjVar("MobileTeamType")

    if( userTeam == nil ) then      
      PlayerRequestTeam(user)
    elseif( userTeam == Warchief.MyTeam ) then
      CollectItems(user)
    else
      this:NpcSpeech("Do that again and you'll draw back a stump!")
    end
end

function HandleEnterView(target)
  --Only talk 50% of the time
  if(math.random(100) > 50) then
      this:NpcSpeech(Warchief.ThingsToSay[math.random(1,#Warchief.ThingsToSay)])
  end
  
  this:NpcSpeech("Do you wish to [FFF544][join][-] "..Warchief.MyTeam.." in its fight?")
  this:SetFacing(this:GetLoc():YAngleTo(target:GetLoc()))
end

this:ScheduleTimerDelay(TimeSpan.FromSeconds(15), "Essence")

RegisterEventHandler(EventType.PlayerSpeech, "", HandlePlayerSpeech)
RegisterEventHandler(EventType.ClientObjectCommand,"dialogResult", HandleDialogResult)
RegisterEventHandler(EventType.ClientObjectCommand, "interact", HandleInteract)
RegisterEventHandler(EventType.EnterView, "NearbyPlayer", HandleEnterView)

AddView("NearbyPlayer", SearchPlayerInRange(5))

